import { world, BlockPermutation, system } from "@minecraft/server";
import { ModalFormData } from "mojang-minecraft-ui";
import { data } from "./vanilla_data"

async function desync(ticks) {
  for (let i = 0; i < ticks; i++) await null;
  return true;
}

world.beforeEvents.itemUseOn.subscribe(async (eventItem) => {
  const { itemStack, source, block } = eventItem;
  if (itemStack.typeId == "minecraft:book") {
    eventItem.cancel = true;
    await desync();
    changePermutationBlock(source, block);
  }
});

/**
 * @param {import("@minecraft/server").Player} player
 * @param {import("@minecraft/server").Block} block
 */
function changePermutationBlock(player, block) {
  if (player.isBusy || player.runCommand("testfor @s[m=c]").successCount == 0) return;  
  const permutate = block.permutation;
  const blockStates = permutate.getAllStates();
  const length = Object.keys(blockStates).length;
  if (player.lastTick == undefined) {
    player.lastTick = 0
  }
  if (length == 0) {
    if (system.currentTick - player.lastTick > 5) {
      player.sendMessage({ "rawtext": [{ "text": "Block " }, { "text": block.type.id }, { "text": " does not have any block states." }] });
      player.lastTick = system.currentTick;
    }
    return;
  };
  player.isBusy = true;
  const permutationForm = new ModalFormData().title(block.type.id);
  for (let state in blockStates) {
    var permutation = blockStates[state]
    //console.warn(state + ": " + permutation + " --> " + typeof (permutation));
    if (!player.isSneaking && state in data) {
      switch (typeof permutation) {
        case "boolean":
          permutationForm.toggle(state, permutation);
          break;
        case "string":
          permutationForm.dropdown(state, data[state], data[state].indexOf(permutation));
          break;
        case "number":
          permutationForm.slider(state, data[state][0], data[state][1], 1, permutation);
          break;
      }
    }
    else {
      switch (typeof permutation) {
        case "boolean":
          permutationForm.toggle(state, permutation);
          break;
        case "string":
          permutationForm.textField(state, permutation);
          break;
        case "number":
          permutationForm.slider(state, 0, 15, 1, permutation);
          break;
      }
    }
  }
  permutationForm.show(player).then((response) => {
    player.isBusy = false;
    var permutation = BlockPermutation.resolve(block.type.id);
    for (let x = 0; x < length; x++) {
      let state = Object.keys(blockStates)[x];
      let value = response.formValues[x];
      console.warn(state, value)
      if (typeof (value) == "string" && value == "") {
        value = blockStates[state];
      }
      else if (typeof blockStates[state] == "string" && typeof value == "number") {
        value = data[state][value]
      }
      permutation = permutation.withState(state, value);
    }
    block.setPermutation(permutation);
  })
}